-- Copyright (c) 2012 Cloudera, Inc. All rights reserved.
--
-- Version 58 of the CMF schema. Migration code to be applied on top of the
-- previous version.
--

update SCHEMA_VERSION set VERSION=58;


-- We're going to be adding configs to the CONFIGS table, so let's temporarily
-- enable auto_increment for the CONFIG_ID column.
alter table CONFIGS
    modify column CONFIG_ID bigint not null auto_increment;

-- Now we find the latest config revision for the service that AMON belongs to.
select @latestConfigRevisionId := MGMT.CONFIG_REVISION_ID
from ROLES AMON, SERVICES MGMT
where AMON.SERVICE_ID = MGMT.SERVICE_ID and
      AMON.ROLE_TYPE = 'ACTIVITYMONITOR';

select @amonRoleId := ROLE_ID from ROLES where ROLE_TYPE='ACTIVITYMONITOR';

-- Now we copy over any role instance-level configs. This cannot be refactored
-- with the role type-level copying below using an OR because we cannot have
-- multiple rows with the same ATTR, SERVICE_ID, and REVISION_ID.
-- The "not exists" at the end is included to prevent exactly that problem.

insert into CONFIGS(ATTR, VALUE, SERVICE_ID, REVISION_ID)
select C1.ATTR, C1.VALUE, MR.SERVICE_ID, MR.CONFIG_REVISION_ID
from CONFIGS C1 cross join SERVICES MR
where C1.REVISION_ID=@latestConfigRevisionId and
      C1.ROLE_ID=@amonRoleId and
      MR.SERVICE_TYPE='MAPREDUCE' and
      C1.ATTR in (
          'firehose_activity_duration_rules',
          'firehose_activity_slow_alert',
          'firehose_activity_failure_alert') and
      not exists(
          select ATTR, SERVICE_ID, REVISION_ID from CONFIGS C2
          where C2.ATTR = C1.ATTR and
                C2.SERVICE_ID = MR.SERVICE_ID and
                C2.REVISION_ID = MR.CONFIG_REVISION_ID);

-- Now we copy over any role type-level configs. This cannot be refactored
-- with the role instance-level copying above (see above for details).

insert into CONFIGS(ATTR, VALUE, SERVICE_ID, REVISION_ID)
select C1.ATTR, C1.VALUE, MR.SERVICE_ID, MR.CONFIG_REVISION_ID
from CONFIGS C1 cross join SERVICES MR
where C1.REVISION_ID=@latestConfigRevisionId and
      C1.ROLE_TYPE='ACTIVITYMONITOR' and
      MR.SERVICE_TYPE='MAPREDUCE' and
      C1.ATTR in (
          'firehose_activity_duration_rules',
          'firehose_activity_slow_alert',
          'firehose_activity_failure_alert') and
      not exists(
          select ATTR, SERVICE_ID, REVISION_ID from CONFIGS C2
          where C2.ATTR = C1.ATTR and
                C2.SERVICE_ID = MR.SERVICE_ID and
                C2.REVISION_ID = MR.CONFIG_REVISION_ID);

-- Now we'll disable auto_increment for the CONFIG_ID column of the CONFIGS table.
alter table CONFIGS
    modify column CONFIG_ID bigint not null;
